//******************************************************************
//* USB-CAN BRIDGE 
//* LOCKHEED MARTIN SPACE DAY 2009
//* LAST UPDATED 04/20/2009
//* CHANGES/ADDITIONS
//******************************************************************
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>
#include <stdlib.h>
#include "ecan.c"
#include "ecan.h"

// Settings for PIC18F4680
__CONFIG(1,IESODIS & FCMDIS & HS);
__CONFIG(2,BORDIS & PWRTEN & WDTDIS);
__CONFIG(3,MCLREN & LPT1DIS & PBDIGITAL);
__CONFIG(4,DEBUGEN & LVPDIS & XINSTDIS & STVREN & BBSIZ1K);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__IDLOC(0420);

#define team  3
#define esc 0x1B
//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void init(void);
void interrupt TIMERS(void);
void lcd_send_nibble( char n ); 
void lcd_gotoxy( char x, char y); 
void lcd_init(void); 
void lcd_send_byte( char address, char n );
void bin2hex(char binchar);
void bin2decimal(char binchar);
void scroll_lidmsg(void);
void putch(char c);
unsigned char recvchar(void);
unsigned char sendchar(char data);
char CharInQueue(void);
void init_EUSART(void);
void reset_tmr(char timer);
void clr_flags(void);
void send_can_msg8(char dest_addr, char cmd, char a,char b ,char c, char d, char e, char f);
void send_can_msg7(char dest_addr, char cmd, char a,char b ,char c, char d, char e);
void send_can_msg6(char dest_addr, char cmd, char a,char b ,char c, char d);
void send_can_msg5(char dest_addr, char cmd, char a,char b ,char c);
void send_can_msg4(char dest_addr, char cmd, char a,char b );
void send_can_msg3(char dest_addr, char cmd, char a);
void send_can_msg2(char dest_addr, char cmd);
//******************************************************************
//*	ECAN DEFINITIONS
//******************************************************************
unsigned long id;
char data[8];
char dataLen;
ECAN_RX_MSG_FLAGS canflags;

char my_addr,dest_addr;
unsigned int my_id;

#define id_motor	0x1A0
#define id_team1	0x1A1
#define id_team2	0x1A2
#define id_team3	0x1A3

#define	addr_motor	0xA0
#define addr_team1	0xA1
#define addr_team2	0xA2
#define addr_team3	0xA3
//******************************************************************
//* ECAN COMMANDS
//******************************************************************
#define	cblink1		0x01
#define	cblink2		0x02
#define	cblinkleds	0x03
#define cblinkoff	0x04
#define cmtr1		0x05
#define cmtr2		0x06
#define cmtr3		0x07
//#define cmsgmode	0x07
//******************************************************************
//*	GLOBALS
//******************************************************************
unsigned int temp16,scratch16;
char temp8,scratch8;
char hundreds, tens, ones;
unsigned int tmsecs1,tmsecs2,tmsecs3;
unsigned int imsecs1,imsecs2,imsecs3;
char tsecs1,tsecs2,tsecs3;
char isecs1,isecs2,isecs3;
char tmins2,tmins3;
char imins2,imins3;

typedef struct 
{
	char blink1:1;
	char blink2:1;
	char cw:1;
	char ccw:1;
	char msgmode:1;
}FFLAGS;

FFLAGS flags;
//******************************************************************
//    MACROS AND PORT/PIN DEFINITIONS
//******************************************************************
#define bitset(var, bitno) ((var) |= 1 << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1 << (bitno)))
#define make8(var,offset)	((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) \
		((unsigned long)var1<<24)+((unsigned long)var2<<16)+ \
		((unsigned long)var3<<8)+((unsigned long)var4)		
#define make32i(var1,var2) ((unsigned long)var1<<16)+((unsigned long)var2)
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR3int  	TMR3IE = 0
#define		enable_TMR3int	 	TMR3IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 0;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 0;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0
#define		TIMER3ON		  	TMR3ON = 1
#define		TIMER3OFF		  	TMR3ON = 0

#define mdelay1(msecdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								TIMER1ON;					\
								while(tmsecs1 < msecdelay);
								
#define	sdelay1(secdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								isecs1 = 0;					\
								tsecs1 = 0;					\
								TIMER1ON;					\
								while(tsecs1 < secdelay);
//******************************************************************
//*	  LCD DEFINITIONS
//******************************************************************
#define databus	   	LATD
#define lcdcntrl   	LATE
#define E 	   		0x01
#define RW  		0x02 
#define RS			0x04 
#define clrRW		lcdcntrl &= ~RW
#define setRW		lcdcntrl |= RW
#define clrRS		lcdcntrl &= ~RS
#define setRS		lcdcntrl |= RS
#define clrE		lcdcntrl &= ~E
#define setE    	lcdcntrl |= E
				   
#define lcdcls	   	lcd_send_byte(0,0x01)
#define	line1	   	lcd_gotoxy(1,1)
#define	line2	   	lcd_gotoxy(2,1)

#define backlight	LATB0

//******************************************************************
//*	CANNED LCD MESSAGES
//******************************************************************
char LCD_INIT_STRING[5] = {0x28,0x08,0x01,0x06,0x0E};
//                        "12345678";
char lcdmsg_team1[]		= " TEAM 1 ";
char lcdmsg_team2[]		= " TEAM 2 ";
char lcdmsg_team3[]		= " TEAM 3 ";
char lcdmsg_motor1[]	= " MOTOR 1";
char lcdmsg_motor2[]	= " MOTOR 2";
char lcdmsg_motor3[]	= " MOTOR 3";
char lcdmsg_ready[]		= " READY! ";
char lcdmsg_1is[]		= "LED1 IS ";
char lcdmsg_blinking[]	= "BLINKING";
char lcdmsg_2is[]		= "LED2 IS ";
char lcdmsg_blinkall[]	= "ALL LEDS";
char lcdmsg_blinkoff[]	= "  OFF   ";
char lcdmsg_sendto[]	= "SEND TO ";
char lcdmsg_blink1[]	= "BLINK 1 ";
char lcdmsg_blink2[]	= "BLINK 2 ";
char lcdmsg_blink12[]	= "BLINK 12";
char lcdmsg_sent[]		= "  SENT  ";
//******************************************************************
//*	GENERAL DEFINITONS
//******************************************************************
#define	LED1		LATC4	
#define	LED2		LATC5
#define	led1_off	LED1 = 1
#define	led1_on		LED1 = 0
#define	led2_off	LED2 = 1
#define	led2_on		LED2 = 0
//******************************************************************
//*	PIC18F4680 RAM Definitions
//******************************************************************
char high_char,low_char,window_bot,window_top;
char buttons,btncode,addr_panel,addr_mtrhtr,displayaddr;
char m8,holdtime,getout;
char bytein,lcdsecs;
//******************************************************************
//*	CLEAR ALL FLAGS
//******************************************************************
void clr_flags(void)
{ 
	flags.blink1 = 0;
	flags.blink2 = 0;
	flags.msgmode = 0;
}
//****************************************************************************
//*	SEND CAN MESSAGES
//****************************************************************************
void send_can_msg8(char dest_addr, char cmd, char a,char b ,char c, char d, char e, char f)
{
	dataLen = 8;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	data[3] = b;
	data[4] = c;
	data[5] = d;
	data[6] = e;
	data[7] = f;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg7(char dest_addr, char cmd, char a,char b ,char c, char d, char e)
{
	dataLen = 7;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	data[3] = b;
	data[4] = c;
	data[5] = d;
	data[6] = e;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg6(char dest_addr, char cmd, char a,char b ,char c, char d)
{
	dataLen = 6;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	data[3] = b;
	data[4] = c;
	data[5] = d;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg5(char dest_addr, char cmd, char a,char b ,char c)
{
	dataLen = 5;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	data[3] = b;
	data[4] = c;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg4(char dest_addr, char cmd, char a,char b)
{
	dataLen = 4;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	data[3] = b;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg3(char dest_addr, char cmd, char a)
{
	dataLen = 3;
	data[0] = dest_addr;
	data[1] = cmd;
	data[2] = a;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
void send_can_msg2(char dest_addr, char cmd)
{
	dataLen = 2;
	data[0] = dest_addr;
	data[1] = cmd;
	while(!ECANSendMessage(id, data, dataLen, ECAN_TX_STD_FRAME));
	mdelay1(10);
}
//******************************************************************
//* RESET TIMERS ROUTINE
//******************************************************************
void reset_tmr(char timer)
{
	switch(timer)
	{
		case 1:
			TIMER1OFF;
			TMR1IF = 0;
			TMR1H = 0xEC;	
			TMR1L = 0x77;
			imsecs1 = 0;
			isecs1 = 0;
			tmsecs1 = 0;
			tsecs1 = 0;
		break;

		case 2:
			TIMER2OFF;
			TMR2IF = 0;
			imsecs2 = 0;
			imins2 = 0;
			isecs2 = 0;
			tmsecs2 = 0;
			tsecs2 = 0;
			tmins2 = 0;
		break;

		case 3:
			TIMER3OFF;
			TMR3IF = 0;
			TMR3H = 0xEC;	
			TMR3L = 0x77;
			imsecs3 = 0;
			isecs3 = 0;
			imins3 = 0;
			tmsecs3 = 0;
			tsecs3 = 0;
			tmins3 = 0;
		break;
	}
}
//******************************************************************
//*	USART BUFFER DEFINITIONS
//******************************************************************
#define BAUD	57600		//desired baud rate
#define FOSC	20000000	//oscillator frequency
#define DIVIDER ((unsigned int)(FOSC/(16 * BAUD) - 1))
//1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_RX_BUFFER_SIZE  16     
#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )
// 1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_TX_BUFFER_SIZE  16      
#define USART_TX_BUFFER_MASK ( USART_TX_BUFFER_SIZE - 1 )
unsigned char USART_RxBuf[USART_RX_BUFFER_SIZE],USART_TxBuf[USART_TX_BUFFER_SIZE];
unsigned char USART_TxHead,USART_TxTail,USART_RxHead,USART_RxTail;

void putch(char c)
{
   while(!TRMT) 	// TRMT is set when TSR is empty
      continue;

  TXREG = c;		// load the register
}//******************************************************************
//*	INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMERS(void)
{
	  unsigned char usart_data,tmphead,tmptail;

	if((TMR3IF && TMR3IE))
	{
		TMR3IF = 0;
		TMR3H = 0xEC;	
		TMR3L = 0x77;

		++tmsecs3;
		if(++imsecs3 == 100)
		{
			imsecs3 = 0;
			if(flags.blink1)
			{
				LED1 ^= 1;
			}
			if(flags.blink2)
			{
				LED2 ^= 1;
			}
		}
	}

	if((TMR2IF && TMR2IE))
	{
		TMR2IF = 0;

		++tmsecs2;
		if(++imsecs2 == 1000)
		{
			imsecs2 = 0;
			++isecs2;
			++tsecs2;
		}
		if(isecs2 == 60)
		{
			isecs2 = 0;
			++imins2;
			++tmins2;
		}
	}

	if((TMR1IF && TMR1IE))
	{
		TMR1IF = 0;
		TMR1H = 0xEC;	
		TMR1L = 0x77;

		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			++isecs1;
			++tsecs1;
		}
	}
  if(RCIF)
 {
   usart_data = RCREG;                   // read the received usart_data 
                                // calculate buffer index 
   tmphead = ( USART_RxHead + 1 ) & USART_RX_BUFFER_MASK;
   USART_RxHead = tmphead;        // store new index 

   if ( tmphead == USART_RxTail )
   {
     // ERROR! Receive buffer overflow 
   }
                                
   USART_RxBuf[tmphead] = usart_data;   // store received usart_data in buffer 
  }

  
  if(TRMT)
  {
                                // check if all usart_data is transmitted  
   if ( USART_TxHead != USART_TxTail )
   {
                                // calculate buffer index  
     tmptail = ( USART_TxTail + 1 ) & USART_TX_BUFFER_MASK;
     USART_TxTail = tmptail;      // store new index  

    TXREG = USART_TxBuf[tmptail];  // start transmitiON  
   }
   else
   {
     TXIE = 0;         // disable TX interrupt  
   }
  }
} 
//******************************************************************
//*	Init USART Function
//******************************************************************
void init_EUSART(void)
{
  SPBRG = 64;//DIVIDER;	//load baud rate divisor-calc didn't work here	
  TRISC7 = 1;			//receive pin
  TRISC6 = 0;			//transmit pin
  TXSTA = 0x04;		//high speed baud rate	
  RCSTA = 0x80;		//enable serial port and serial port pins
  
  USART_RxTail = 0x00;	//flush receive buffer 
  USART_RxHead = 0x00;
  USART_TxTail = 0x00;	//flush transmit buffer
  USART_TxHead = 0x00;
  RCIP = 1;			//receive interrupt = high priority
  TXIP = 1;			//transmit interrupt = high priority
  RCIE = 1;			//enable receive interrupt
  PEIE = 1;				//enable all unmasked peripheral interrupts
  GIE = 1;				//enable all unmasked interrupts
  CREN = 1;			//enable USART1 receiver
  TXIE = 0;			//disable USART1 transmit interrupt
  TXEN = 1;			//transmitter enabled
}

unsigned char recvchar(void)
{
  unsigned char tmptail;
                                // wait for incomming usart_data  
  while ( USART_RxHead == USART_RxTail );
                                // calculate buffer index  
  tmptail = ( USART_RxTail + 1 ) & USART_RX_BUFFER_MASK;
  USART_RxTail = tmptail;        // store new index  

  return USART_RxBuf[tmptail];   // return usart_data  
}

unsigned char sendchar(char usart_data)
{
  unsigned char tmphead;
                                // calculate buffer index 
  tmphead = ( USART_TxHead + 1 ) & USART_TX_BUFFER_MASK;
                                // wait for free space in buffer 
  while ( tmphead == USART_TxTail );
                                // store usart_data in buffer 
  //USART_TxBuf[tmphead] = (unsigned char)usart_data;
  USART_TxBuf[tmphead] = usart_data;
  USART_TxHead = tmphead;        // store new index 

  TXIE = 1;            // enable TX interrupt 

  return usart_data;
}

unsigned char CharInQueue(void)
{
  return(USART_RxHead != USART_RxTail);
}




//****************************************************************************

//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2hex(char binchar)
{
   char high_nibble,low_nibble;

   high_nibble = (binchar & 0xF0) / 16;
   if(high_nibble > 0x09)
      high_char = high_nibble + 0x37;
   else
      high_char = high_nibble + 0x30;

   low_nibble = (binchar & 0x0F);
   if(low_nibble > 0x09)
      low_char = low_nibble + 0x37;
   else
      low_char = low_nibble + 0x30;
}
//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2decimal(char binchar)
{
	high_char = 0x30;
	low_char = 0x30;
	if(binchar >= 10)
	{
   		do{
			binchar -= 10;
			++high_char;
		}while(binchar >= 10);
	}
	low_char = low_char + binchar;
}
//******************************************************************
//*	LCD Functions
//******************************************************************
void lcd_send_nibble( char n ) 
{
	databus &= 0x0F;
	databus |= (n & 0xF0);
	mdelay1(1);
	setE;
	mdelay1(1);
	clrE;
}

void lcd_send_byte( char address, char n ) 
{
      clrE;
      
	  switch (address)
	  {
	  		 case 0:
			 	  clrRS;
				  break;
			 case 1:
			 	  setRS;
				  break;
			 default:
			 	  setRS;
				  break;	
      }		
	  		mdelay1(1);
		    
      lcd_send_nibble (n);
      lcd_send_nibble (n << 4);
	  		mdelay1(1);

}

void lcd_init(void) 
{
    char j8;
    clrRS;
    clrE;
    clrRW;
	mdelay1(16);
	lcd_send_nibble(0x30);
	mdelay1(6);
	lcd_send_nibble(0x30);
	mdelay1(1);
	lcd_send_nibble(0x30);
	mdelay1(2);
    lcd_send_nibble(0x20);
	for(j8=0;j8<5;++j8)
	{
	mdelay1(2);
       lcd_send_byte(0,LCD_INIT_STRING[j8]);
	}   
}

void lcd_gotoxy( char x, char y) 
{
// where x = lcd row (1,2,3,4) and y = column (1 thru 20)
   char address;
   switch (x)
   {
   		  case 1:
		  	   address = 0;
			   break;
		  case 2:
		  	   address = 0x40;
			   break;
		  case 3:
		  	   address = 0x14;
			   break;
		  case 4:
		  	   address = 0x54;
			   break;
		  default:
		  	   address = 0;
   }   
   address += (y-1);
   lcd_send_byte(0,0x80|address);
}
//******************************************************************
//*	INITIALIZE
//******************************************************************
void init(void)
{
	char init8;
	
   	TRISA = 0xFF;		
   	TRISB = 0b11101010;		
   	TRISC = 0b11001101;	
   	TRISD = 0x00;		
	TRISE = 0b00000000;
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ADCON0 = 0b00000000;	//turn off A2D
	ADCON1 = 0b00001111;	//PORTA all digital
	CMCON = 0x07;			//comparators off
//******************************************************************
//*	INITIALIZE CAN HARDWARE
//******************************************************************
	switch(team)
	{
		case 1:
			my_id = id_team1;
			my_addr = addr_team1;
		break;
		case 2:
			my_id = id_team2;
			my_addr = addr_team2;
		break;
		case 3:
			my_id = id_team3;
			my_addr = addr_team3;
		break;
	}
	ECANInitialize();
//******************************************************************
//*	INITIALIZE EUSART
//******************************************************************
	init_EUSART();
//******************************************************************
//*	CONFIGURE AND START TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	T1CON = 0b00000000;
	TMR1H = 0xEC;
	TMR1L = 0x77;
	TIMER1ON;
//******************************************************************
//*	CONFIGURE AND START TIMER2
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER2OFF;
	T2CON = 0x79;		
	PR2 = 0x4F;	
	TIMER2ON;
//******************************************************************
//*	CONFIGURE AND START TIMER3
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	imins3 = 0;
	isecs3 = 0;
  	imsecs3 = 0;
	T3CON = 0x00;		
	TMR3H = 0xEC;	
	TMR3L = 0x77;
	TIMER3ON;
//******************************************************************
//*	CONFIGURE INTERRUPTS
//******************************************************************
	clr_flags();
	led1_off;
	led2_off;
	enable_TMR1int;
	enable_TMR2int;
	enable_TMR3int;
    enable_GLOBALint;	
//******************************************************************
//*	INITIALIZE GLOBAL VARIABLES
//******************************************************************
//******************************************************************
//*	INITIALIZE LCD
//******************************************************************
		lcd_init();
		backlight = 1;
		lcdcls;
		line1;
		switch(my_addr)
		{
			case addr_team1:
				for(init8=0;init8<8;++init8)
					lcd_send_byte(1,lcdmsg_team1[init8]);
			break;
			case addr_team2:
				for(init8=0;init8<8;++init8)
					lcd_send_byte(1,lcdmsg_team2[init8]);
			break;
			case addr_team3:
				for(init8=0;init8<8;++init8)
					lcd_send_byte(1,lcdmsg_team3[init8]);
			break;
		}
				
		line2;
			for(init8=0;init8<8;++init8)
				lcd_send_byte(1,lcdmsg_ready[init8]);
}
	
void main(void)
{
//******************************************************************
//*	INITIALIZE
//******************************************************************
  init();
//******************************************************************
//*	MAIN SERVICE LOOP
//******************************************************************

//******************************************************************
//*	CAN SERVICE LOOP
//******************************************************************
	do{
		if( ECANReceiveMessage(&id, data, &dataLen, &canflags) )
		{
		   	if (data[0] == my_addr)
		   	{
				  	switch(data[1])
					{
						case cblink1:
							clr_flags();
							led2_off;
							flags.blink1 = 1;
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_1is[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinking[m8]);
						break;
						case cblink2:
							clr_flags();
							led1_off;
							flags.blink2 = 1;
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_2is[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinking[m8]);
						break;
						case cblinkleds:
							clr_flags();
							led1_off;
							led2_on;
							flags.blink2 = 1;
							flags.blink1 = 1;
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkall[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinking[m8]);
						break;
						case cblinkoff:
							clr_flags();
							led1_off;
							led2_off;
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkall[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkoff[m8]);
						break;
						default:
							sendchar(data[1]);
						break;

					}
				}
			
		}
							
//******************************************************************
//*	VISUAL BASIC SERIAL CHARACTER SERVICE LOOP
//******************************************************************
		if( CharInQueue() )
		{
				
			bytein = recvchar();
			
				switch(bytein)
				{
					case 0x01:
						dest_addr = addr_team1;
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_team1[m8]);
					break;
					case 0x02:
						dest_addr = addr_team2;
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_team2[m8]);
					break;
					case 0x03:
						dest_addr = addr_team3;
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_team3[m8]);
					break;
					case 0x04:
						if(my_addr == dest_addr)
						{
							clr_flags();
							led2_off;
							flags.blink1 = 1;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_1is[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_blinking[m8]);
						}
						else
						{
							send_can_msg2(dest_addr,cblink1);
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_sent[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blink1[m8]);
						}
					break;
					case 0x05:
						if(my_addr == dest_addr)
						{
							clr_flags();
							led1_off;
							flags.blink2 = 1;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_2is[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinking[m8]);
						}
						else
						{
							send_can_msg2(dest_addr,cblink2);
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_sent[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blink2[m8]);
						}
					break;
					case 0x06:
						if(my_addr == dest_addr)
						{
							clr_flags();
							led1_off;
							led2_on;
							flags.blink2 = 1;
							flags.blink1 = 1;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkall[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinking[m8]);
						}
						else
						{
							send_can_msg2(dest_addr,cblinkleds);
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_sent[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blink12[m8]);
						}
					break;
					case 0x07:
						if(my_addr == dest_addr)
						{
							clr_flags();
							led1_off;
							led2_off;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkall[m8]);
							line2;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_blinkoff[m8]);
						}
						else
						{
							send_can_msg2(dest_addr,cblinkoff);
							lcdcls;
							line1;
							for(m8=0;m8<8;++m8)
								lcd_send_byte(1,lcdmsg_sent[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_blinkoff[m8]);
						}
					break;
					case 0x08:
						send_can_msg2(addr_motor,cmtr1);
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_motor1[m8]);
					break;
					case 0x09:
						send_can_msg2(addr_motor,cmtr2);
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_motor2[m8]);
					break;
					case 0x0B:
						send_can_msg2(addr_motor,cmtr3);
						lcdcls;
						line1;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_sendto[m8]);
						line2;
						for(m8=0;m8<8;++m8)
							lcd_send_byte(1,lcdmsg_motor3[m8]);
					break;
					case 0x0C:
							lcdcls;
							line1;
							switch(my_addr)
							{
								case addr_team1:
									for(m8=0;m8<8;++m8)
										lcd_send_byte(1,lcdmsg_team1[m8]);
								break;
								case addr_team2:
									for(m8=0;m8<8;++m8)
										lcd_send_byte(1,lcdmsg_team2[m8]);
								break;
								case addr_team3:
									for(m8=0;m8<8;++m8)
										lcd_send_byte(1,lcdmsg_team3[m8]);
								break;
							}
									
							line2;
								for(m8=0;m8<8;++m8)
									lcd_send_byte(1,lcdmsg_ready[m8]);
					break;

					default:
						send_can_msg2(dest_addr, bytein);
					break;
				}
				
	 	}
	}while(1);
}
	
	
	
	
